@@ -62,10 +62,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "compiler.h"
 
-/* Drivers using the mi implementation of backing store need: */
-
-#include "mibstore.h"
-
 /* All drivers using the vgahw module need this */
 /* This driver needs to be modified to not use vgaHW for multihead operation */
 #include "vgaHW.h"
@@ -124,28 +120,28 @@ static Bool TDFXProbe(DriverPtr drv, int flags);
 static Bool TDFXPreInit(ScrnInfoPtr pScrn, int flags);
 
 /* Initialize a screen */
-static Bool TDFXScreenInit(int Index, ScreenPtr pScreen, int argc, char **argv);
+static Bool TDFXScreenInit(SCREEN_INIT_ARGS_DECL);
 
 /* Enter from a virtual terminal */
-static Bool TDFXEnterVT(int scrnIndex, int flags);
+static Bool TDFXEnterVT(VT_FUNC_ARGS_DECL);
 
 /* Leave to a virtual terminal */
-static void TDFXLeaveVT(int scrnIndex, int flags);
+static void TDFXLeaveVT(VT_FUNC_ARGS_DECL);
 
 /* Close down each screen we initialized */
-static Bool TDFXCloseScreen(int scrnIndex, ScreenPtr pScreen);
+static Bool TDFXCloseScreen(CLOSE_SCREEN_ARGS_DECL);
 
 /* Change screensaver state */
 static Bool TDFXSaveScreen(ScreenPtr pScreen, int mode);
 
 /* Cleanup server private data */
-static void TDFXFreeScreen(int scrnIndex, int flags);
+static void TDFXFreeScreen(FREE_SCREEN_ARGS_DECL);
 
 /* Check if a mode is valid on the hardware */
-static ModeStatus TDFXValidMode(int scrnIndex, DisplayModePtr mode,
+static ModeStatus TDFXValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode,
 				Bool verbose, int flags);
 
-static void TDFXBlockHandler(int, pointer, pointer, pointer);
+static void TDFXBlockHandler(BLOCKHANDLER_ARGS_DECL);
 
 /* Switch to various Display Power Management System levels */
 static void TDFXDisplayPowerManagementSet(ScrnInfoPtr pScrn, 
@@ -153,17 +149,20 @@ static void TDFXDisplayPowerManagementSet(ScrnInfoPtr pScrn,
 
 #ifdef XSERVER_LIBPCIACCESS
 #define TDFX_DEVICE_MATCH(d, sub, i) \
-    { 0x121A, (d), PCI_MATCH_ANY, (sub), 0, 0, (i) }
+    { PCI_VENDOR_3DFX, (d), PCI_MATCH_ANY, (sub), 0, 0, (i) }
 
 static const struct pci_id_match tdfx_device_match[] = {
     TDFX_DEVICE_MATCH(PCI_CHIP_BANSHEE, PCI_MATCH_ANY, Banshee),
 
+    TDFX_DEVICE_MATCH(PCI_CHIP_VELOCITY, PCI_MATCH_ANY, Voodoo3_Unknown),
+
     /* There are *many* missing PCI IDs here.
      */
-    TDFX_DEVICE_MATCH(PCI_CHIP_VOODOO3, 0x0036, Voodoo3_2000),
-    TDFX_DEVICE_MATCH(PCI_CHIP_VOODOO3, 0x003A, Voodoo3_3000),
+    TDFX_DEVICE_MATCH(PCI_CHIP_VOODOO3, PCI_CARD_VOODOO3_2000, Voodoo3_2000),
+    TDFX_DEVICE_MATCH(PCI_CHIP_VOODOO3, PCI_CARD_VOODOO3_3000, Voodoo3_3000),
 
     TDFX_DEVICE_MATCH(PCI_CHIP_VOODOO3, PCI_MATCH_ANY, Voodoo3_Unknown),
+    TDFX_DEVICE_MATCH(PCI_CHIP_VOODOO4, PCI_MATCH_ANY, Voodoo5),
     TDFX_DEVICE_MATCH(PCI_CHIP_VOODOO5, PCI_MATCH_ANY, Voodoo5),
     { 0, 0, 0 }
 };
@@ -202,7 +201,9 @@ _X_EXPORT DriverRec TDFX = {
 /* Chipsets */
 static SymTabRec TDFXChipsets[] = {
   { PCI_CHIP_BANSHEE, "3dfx Banshee"},
+  { PCI_CHIP_VELOCITY, "3dfx Velocity"},
   { PCI_CHIP_VOODOO3, "3dfx Voodoo3"},
+  { PCI_CHIP_VOODOO4, "3dfx Voodoo4"},
   { PCI_CHIP_VOODOO5, "3dfx Voodoo5"},
   { -1, NULL }
 };
@@ -210,7 +211,9 @@ static SymTabRec TDFXChipsets[] = {
 #ifndef XSERVER_LIBPCIACCESS
 static PciChipsets TDFXPciChipsets[] = {
   { PCI_CHIP_BANSHEE, PCI_CHIP_BANSHEE, RES_SHARED_VGA },
+  { PCI_CHIP_VELOCITY, PCI_CHIP_VELOCITY, RES_SHARED_VGA },
   { PCI_CHIP_VOODOO3, PCI_CHIP_VOODOO3, RES_SHARED_VGA },
+  { PCI_CHIP_VOODOO4, PCI_CHIP_VOODOO4, RES_SHARED_VGA },
   { PCI_CHIP_VOODOO5, PCI_CHIP_VOODOO5, RES_SHARED_VGA },
   { -1, -1, RES_UNDEFINED }
 };
@@ -633,30 +636,30 @@ TDFXInitChips(ScrnInfoPtr pScrn)
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, 3,
 		   "TDFXInitChips: numchips = %d\n", pTDFX->numChips);
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, 3,
-		   "TDFXInitChips: cfgbits = 0x%08lx, initbits = 0x%08lx\n",
+		   "TDFXInitChips: cfgbits = 0x%08x, initbits = 0x%08x\n",
 		   cfgbits, initbits);
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, 3,
-		   "TDFXInitChips: mem0base = 0x%08lx, mem1base = 0x%08lx\n",
+		   "TDFXInitChips: mem0base = 0x%08x, mem1base = 0x%08x\n",
 		   mem0base, mem1base);
 
     mem0size = 32 * 1024 * 1024; /* Registers are always 32MB */
     mem1size = pScrn->videoRam * 1024 * 2; /* Linear mapping is 2x memory */
 
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, 3,
-		   "TDFXInitChips: mem0size = 0x%08lx, mem1size = 0x%08lx\n",
+		   "TDFXInitChips: mem0size = 0x%08x, mem1size = 0x%08x\n",
 		   mem0size, mem1size);
 
     mem0bits = TDFXSizeToCfg(mem0size);
     mem1bits = TDFXSizeToCfg(mem1size) << 4;
 
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, 3,
-		   "TDFXInitChips: mem0bits = 0x%08lx, mem1bits = 0x%08lx\n",
+		   "TDFXInitChips: mem0bits = 0x%08x, mem1bits = 0x%08x\n",
 		   mem0bits, mem1bits);
 
     cfgbits = (cfgbits & ~(0xFF)) | mem0bits | mem1bits;
 
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, 3,
-		   "TDFXInitChips: cfgbits = 0x%08lx\n", cfgbits);
+		   "TDFXInitChips: cfgbits = 0x%08x\n", cfgbits);
 
     for (i = 0; i < pTDFX->numChips; i++) {
 	PCI_WRITE_LONG(initbits | BIT(10), CFG_INIT_ENABLE, i);
@@ -750,14 +753,13 @@ Bool TDFXI2cInit(ScrnInfoPtr pScrn)
 static xf86MonPtr doTDFXDDC(ScrnInfoPtr pScrn)
 {
   TDFXPtr pTDFX = TDFXPTR(pScrn);
-  I2CBusPtr pI2CBus;
   xf86MonPtr pMon = NULL;
   CARD32 reg;
 
   reg = pTDFX->readLong(pTDFX, VIDSERIALPARALLELPORT);
   pTDFX->writeLong(pTDFX, VIDSERIALPARALLELPORT, reg | VSP_ENABLE_IIC0);
 
-  pMon = xf86DoEDID_DDC2(pScrn->scrnIndex, pTDFX->pI2CBus);
+  pMon = xf86DoEDID_DDC2(XF86_SCRN_ARG(pScrn), pTDFX->pI2CBus);
 
   if (pMon == NULL)
     xf86Msg(X_WARNING, "No DDC2 capable monitor found\n");
@@ -1054,6 +1056,7 @@ TDFXPreInit(ScrnInfoPtr pScrn, int flags)
     case PCI_CHIP_BANSHEE:
       pTDFX->MaxClock = 270000;
       break;
+    case PCI_CHIP_VELOCITY:
     case PCI_CHIP_VOODOO3:
       switch(match->subsysCard) {
       case PCI_CARD_VOODOO3_2000:
@@ -1067,6 +1070,7 @@ TDFXPreInit(ScrnInfoPtr pScrn, int flags)
 	break;
       }
       break;
+    case PCI_CHIP_VOODOO4:
     case PCI_CHIP_VOODOO5:
       pTDFX->MaxClock = 350000;
       break;
@@ -1082,7 +1086,9 @@ TDFXPreInit(ScrnInfoPtr pScrn, int flags)
     case PCI_CHIP_BANSHEE:
       clockRanges->interlaceAllowed = FALSE;
       break;
+    case PCI_CHIP_VELOCITY:
     case PCI_CHIP_VOODOO3:
+    case PCI_CHIP_VOODOO4:
     case PCI_CHIP_VOODOO5:
       clockRanges->interlaceAllowed = TRUE;
       break;
@@ -1257,11 +1263,11 @@ TDFXMapMem(ScrnInfoPtr pScrn)
      * FIXME: don't have any such hardware to test.
      */
     for (i = 0; i < pTDFX->numChips; i++) {
-	err = pci_device_map_memory_range(pTDFX->PciInfo[i],
-					  pTDFX->MMIOAddr[i],
-					  TDFXIOMAPSIZE,
-					  TRUE,
-					  & pTDFX->MMIOBase[i]);
+	err = pci_device_map_range(pTDFX->PciInfo[i],
+				   pTDFX->MMIOAddr[i],
+				   TDFXIOMAPSIZE,
+				   PCI_DEV_MAP_FLAG_WRITABLE,
+				   & pTDFX->MMIOBase[i]);
 	if (err) {
 	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 		       "Unable to map MMIO region for card %u (%d).\n",
@@ -1271,11 +1277,11 @@ TDFXMapMem(ScrnInfoPtr pScrn)
     }
     
 
-    err = pci_device_map_memory_range(pTDFX->PciInfo[0],
-				      pTDFX->LinearAddr[0],
-				      pTDFX->FbMapSize,
-				      TRUE,
-				      & pTDFX->FbBase);
+    err = pci_device_map_range(pTDFX->PciInfo[0],
+			       pTDFX->LinearAddr[0],
+			       pTDFX->FbMapSize,
+			       PCI_DEV_MAP_FLAG_WRITABLE,
+			       & pTDFX->FbBase);
     if (err) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 		   "Unable to map framebuffer (%d).\n", err);
@@ -1311,8 +1317,15 @@ TDFXUnmapMem(ScrnInfoPtr pScrn)
   pTDFX = TDFXPTR(pScrn);
 
 #ifdef XSERVER_LIBPCIACCESS
-    pci_device_unmap_region(pTDFX->PciInfo[0], 0);
-    pci_device_unmap_region(pTDFX->PciInfo[0], 1);
+    pci_device_unmap_range(pTDFX->PciInfo[0],
+                           pTDFX->FbBase,
+                           pTDFX->FbMapSize);
+
+    for (i = 0; i < pTDFX->numChips; i++) {
+        pci_device_unmap_range(pTDFX->PciInfo[i],
+                               pTDFX->MMIOBase[i],
+                               TDFXIOMAPSIZE);
+    }
 
     (void) memset(pTDFX->MMIOBase, 0, sizeof(pTDFX->MMIOBase));
     pTDFX->FbBase = NULL;
@@ -1910,14 +1923,14 @@ TDFXModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 
 #ifdef TDFXDRI
   if (pTDFX->directRenderingEnabled) {
-    DRILock(screenInfo.screens[pScrn->scrnIndex], 0);
-    TDFXSwapContextFifo(screenInfo.screens[pScrn->scrnIndex]);
+    DRILock(xf86ScrnToScreen(pScrn), 0);
+    TDFXSwapContextFifo(xf86ScrnToScreen(pScrn));
   }
 #endif
   DoRestore(pScrn, &hwp->ModeReg, &pTDFX->ModeReg, FALSE);
 #ifdef TDFXDRI
   if (pTDFX->directRenderingEnabled) {
-    DRIUnlock(screenInfo.screens[pScrn->scrnIndex]);
+    DRIUnlock(xf86ScrnToScreen(pScrn));
   }
 #endif
 
@@ -2195,19 +2208,16 @@ static void allocateMemory(ScrnInfoPtr pScrn) {
 }
 
 static Bool
-TDFXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
+TDFXScreenInit(SCREEN_INIT_ARGS_DECL) {
   ScrnInfoPtr pScrn;
   vgaHWPtr hwp;
   TDFXPtr pTDFX;
   VisualPtr visual;
   BoxRec MemBox;
-#ifdef TDFXDRI
-  MessageType driFrom = X_DEFAULT;
-#endif
   int scanlines;
 
   TDFXTRACE("TDFXScreenInit start\n");
-  pScrn = xf86Screens[pScreen->myNum];
+  pScrn = xf86ScreenToScrn(pScreen);
   pTDFX = TDFXPTR(pScrn);
   hwp = VGAHWPTR(pScrn);
 
@@ -2264,7 +2274,7 @@ TDFXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
   }
 
   scanlines = (pTDFX->backOffset - pTDFX->fbOffset) / pTDFX->stride;
-  if(pTDFX->ChipType < PCI_CHIP_VOODOO5) {
+  if(pTDFX->ChipType < PCI_CHIP_VOODOO4) {
       if (scanlines > 2047) 
 	scanlines = 2047;
   } else {
@@ -2311,15 +2321,12 @@ TDFXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
 #ifdef TDFXDRI
   /*
    * Setup DRI after visuals have been established, but before fbScreenInit
-   * is called.   fbScreenInit will eventually call into the drivers
-   * InitGLXVisuals call back.
+   * is called.
    */
   if (!xf86ReturnOptValBool(pTDFX->Options, OPTION_DRI, TRUE) || pTDFX->NoAccel) {
       pTDFX->directRenderingEnabled = FALSE;
-      driFrom = X_CONFIG;
   } else if (pTDFX->texSize < 0) {
       pTDFX->directRenderingEnabled = FALSE;
-      driFrom = X_PROBED;
   } else {
       pTDFX->directRenderingEnabled = TDFXDRIScreenInit(pScreen);
   }
@@ -2337,7 +2344,7 @@ TDFXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
       return FALSE;
     break;
   default:
-    xf86DrvMsg(scrnIndex, X_ERROR,
+    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 	       "Internal error: invalid bpp (%d) in TDFXScrnInit\n",
 	       pScrn->bitsPerPixel);
     return FALSE;
@@ -2373,7 +2380,6 @@ TDFXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
     }
   }
 
-  miInitializeBackingStore(pScreen);
   xf86SetBackingStore(pScreen);
   xf86SetSilkenMouse(pScreen);
 
@@ -2398,7 +2404,7 @@ TDFXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
       return FALSE;
   }
 
-  TDFXAdjustFrame(scrnIndex, 0, 0, 0);
+  TDFXAdjustFrame(ADJUST_FRAME_ARGS(pScrn, 0, 0));
 
   xf86DPMSInit(pScreen, TDFXDisplayPowerManagementSet, 0);
 
@@ -2438,22 +2444,20 @@ TDFXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
 }
 
 Bool
-TDFXSwitchMode(int scrnIndex, DisplayModePtr mode, int flags) {
-  ScrnInfoPtr pScrn;
+TDFXSwitchMode(SWITCH_MODE_ARGS_DECL) {
+  SCRN_INFO_PTR(arg);
 
   TDFXTRACE("TDFXSwitchMode start\n");
-  pScrn=xf86Screens[scrnIndex];
   return TDFXModeInit(pScrn, mode);
 }
 
 void
-TDFXAdjustFrame(int scrnIndex, int x, int y, int flags) {
-  ScrnInfoPtr pScrn;
+TDFXAdjustFrame(ADJUST_FRAME_ARGS_DECL) {
+  SCRN_INFO_PTR(arg);
   TDFXPtr pTDFX;
   TDFXRegPtr tdfxReg;
 
   TDFXTRACE("TDFXAdjustFrame start\n");
-  pScrn = xf86Screens[scrnIndex];
   pTDFX = TDFXPTR(pScrn);
 
   if (pTDFX->ShowCache && y && pScrn->vtSema)
@@ -2468,16 +2472,15 @@ TDFXAdjustFrame(int scrnIndex, int x, int y, int flags) {
 }
 
 static Bool
-TDFXEnterVT(int scrnIndex, int flags) {
-  ScrnInfoPtr pScrn;
+TDFXEnterVT(VT_FUNC_ARGS_DECL) {
+  SCRN_INFO_PTR(arg);
   ScreenPtr pScreen;
 #ifdef TDFXDRI
   TDFXPtr pTDFX;
 #endif
 
   TDFXTRACE("TDFXEnterVT start\n");
-  pScrn = xf86Screens[scrnIndex];
-  pScreen = screenInfo.screens[scrnIndex];
+  pScreen = xf86ScrnToScreen(pScrn);
   TDFXInitFifo(pScreen);
 #ifdef TDFXDRI
   pTDFX = TDFXPTR(pScrn);
@@ -2486,23 +2489,22 @@ TDFXEnterVT(int scrnIndex, int flags) {
   }
 #endif
   if (!TDFXModeInit(pScrn, pScrn->currentMode)) return FALSE;
-  TDFXAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+  TDFXAdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
   return TRUE;
 }
 
 static void
-TDFXLeaveVT(int scrnIndex, int flags) {
-  ScrnInfoPtr pScrn;
+TDFXLeaveVT(VT_FUNC_ARGS_DECL) {
+  SCRN_INFO_PTR(arg);
   vgaHWPtr hwp;
   ScreenPtr pScreen;
   TDFXPtr pTDFX;
 
   TDFXTRACE("TDFXLeaveVT start\n");
-  pScrn = xf86Screens[scrnIndex];
   hwp=VGAHWPTR(pScrn);
   TDFXRestore(pScrn);
   vgaHWLock(hwp);
-  pScreen = screenInfo.screens[scrnIndex];
+  pScreen = xf86ScrnToScreen(pScrn);
   pTDFX = TDFXPTR(pScrn);
   pTDFX->sync(pScrn);
   TDFXShutdownFifo(pScreen);
@@ -2514,14 +2516,14 @@ TDFXLeaveVT(int scrnIndex, int flags) {
 }
 
 static Bool
-TDFXCloseScreen(int scrnIndex, ScreenPtr pScreen)
+TDFXCloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
   ScrnInfoPtr pScrn;
   vgaHWPtr hwp;
   TDFXPtr pTDFX;
 
   TDFXTRACE("TDFXCloseScreen start\n");
-  pScrn = xf86Screens[scrnIndex];
+  pScrn = xf86ScreenToScrn(pScreen);
   hwp = VGAHWPTR(pScrn);
   pTDFX = TDFXPTR(pScrn);
 
@@ -2541,8 +2543,10 @@ TDFXCloseScreen(int scrnIndex, ScreenPtr pScreen)
       vgaHWUnmapMem(pScrn);
   }
   
+#ifdef HAVE_XAA_H
   if (pTDFX->AccelInfoRec) XAADestroyInfoRec(pTDFX->AccelInfoRec);
   pTDFX->AccelInfoRec=0;
+#endif
   if (pTDFX->DGAModes) free(pTDFX->DGAModes);
   pTDFX->DGAModes=0;
   if (pTDFX->scanlineColorExpandBuffers[0])
@@ -2562,40 +2566,39 @@ TDFXCloseScreen(int scrnIndex, ScreenPtr pScreen)
 
   pScreen->BlockHandler = pTDFX->BlockHandler;
   pScreen->CloseScreen = pTDFX->CloseScreen;
-  return (*pScreen->CloseScreen)(scrnIndex, pScreen);
+  return (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 }
 
 static void
-TDFXFreeScreen(int scrnIndex, int flags) {
+TDFXFreeScreen(FREE_SCREEN_ARGS_DECL) {
+  SCRN_INFO_PTR(arg);
   TDFXTRACE("TDFXFreeScreen start\n");
-  TDFXFreeRec(xf86Screens[scrnIndex]);
+  TDFXFreeRec(pScrn);
   if (xf86LoaderCheckSymbol("vgaHWFreeHWRec"))
-    vgaHWFreeHWRec(xf86Screens[scrnIndex]);
+    vgaHWFreeHWRec(pScrn);
 }
 
 static ModeStatus
-TDFXValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags) {
-  ScrnInfoPtr pScrn;
+TDFXValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose, int flags) {
+  SCRN_INFO_PTR(arg);
   TDFXPtr pTDFX;
 
   TDFXTRACE("TDFXValidMode start\n");
   if ((mode->HDisplay>2048) || (mode->VDisplay>1536)) 
     return MODE_BAD;
   /* Banshee doesn't support interlace, but Voodoo 3 and higher do. */
-  pScrn = xf86Screens[scrnIndex];
   pTDFX = TDFXPTR(pScrn);
   if (mode->Flags&V_INTERLACE) {
     switch (pTDFX->ChipType) {
       case PCI_CHIP_BANSHEE:
         return MODE_BAD;
-        break;
+      case PCI_CHIP_VELOCITY:
       case PCI_CHIP_VOODOO3:
+      case PCI_CHIP_VOODOO4:
       case PCI_CHIP_VOODOO5:
         return MODE_OK;
-        break;
       default:
         return MODE_BAD;
-        break;
     }
   }
   /* In clock doubled mode widths must be divisible by 16 instead of 8 */
@@ -2631,7 +2634,7 @@ TDFXBlankScreen(ScrnInfoPtr pScrn, Bool unblank)
 static Bool
 TDFXSaveScreen(ScreenPtr pScreen, int mode)
 {
-  ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+  ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
   Bool unblank;
 
   TDFXTRACE("TDFXSaveScreen start\n");
@@ -2648,14 +2651,14 @@ TDFXSaveScreen(ScreenPtr pScreen, int mode)
 }                                                                             
 
 static void
-TDFXBlockHandler(int i, pointer blockData, pointer pTimeout, pointer pReadmask)
+TDFXBlockHandler(BLOCKHANDLER_ARGS_DECL)
 {
-    ScreenPtr   pScreen = screenInfo.screens[i];
-    ScrnInfoPtr pScrn   = xf86Screens[i];
+    SCREEN_PTR(arg);
+    ScrnInfoPtr pScrn   = xf86ScreenToScrn(pScreen);
     TDFXPtr     pTDFX   = TDFXPTR(pScrn);
 
     pScreen->BlockHandler = pTDFX->BlockHandler;
-    (*pScreen->BlockHandler) (i, blockData, pTimeout, pReadmask);
+    (*pScreen->BlockHandler) (BLOCKHANDLER_ARGS);
     pScreen->BlockHandler = TDFXBlockHandler;
 
     if(pTDFX->VideoTimerCallback) {
